import 'package:flutter/services.dart';

class FlutterBaseBanner {
  late String _channelNm;
  late MethodChannel _platform;

  var onLoadAdCallback;
  var onFailAdCallback;

  FlutterBaseBanner({
    required String channelName
  }){
    this._channelNm = channelName;
    _initChannel(_channelNm);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
    await _platform.invokeMethod('makeAdView');
  }

  Future<void> load(int y) async {
    await _platform.invokeMethod('load', {"y":y});
  }

  Future<void> loadWidget() async {
    await _platform.invokeMethod('loadWidget');
  }

  Future<void> setInterval(int sec) async {
    await _platform.invokeMethod('setInterval', {"sec":sec});
  }

  Future<void> setCUID(String cuid) async {
    await _platform.invokeMethod('setCUID', cuid);
  }

  Future<void> setAdInfo(String zoneId) async {
    await _platform.invokeMethod('setAdInfo', zoneId);
  }

  Future<void> hideBanner() async {
    await _platform.invokeMethod('hideBanner');
  }

  Future<void> showBanner() async {
    await _platform.invokeMethod('showBanner');
  }

  Future<void> removeBanner() async {
    await _platform.invokeMethod('removeBanner');
  }

  Future<void> setCallbackListener({
    void Function(String zoneId)? onLoadAd,
    void Function(String zoneId)? onFailAd
  }) async {

    if(onLoadAd != null){
      onLoadAdCallback = onLoadAd;
    }
    if(onFailAd != null){
      onFailAdCallback = onFailAd;
    }
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    String zoneId = call.arguments.toString();
    switch (call.method) {
      case 'onLoadAd':
        if (onLoadAdCallback != null) {
          onLoadAdCallback(zoneId);
        };
        break;
      case 'onFailAd':
        if (onFailAdCallback != null) {
          onFailAdCallback(zoneId);
        };
        break;
      default:
        throw ('method not defined');
    }
  }
}